@@ -122,7 +122,7 @@ impl ReadOnlyCacheBuilder {
     ///
     /// Kismet will propagate the error on mismatch.
     pub fn consistency_checker(
-        self,
+        &mut self,
         checker: impl Fn(&mut File, &mut File) -> Result<()>
             + Sync
             + Send
@@ -130,12 +130,28 @@ impl ReadOnlyCacheBuilder {
             + std::panic::UnwindSafe
             + Sized
             + 'static,
-    ) -> Self {
+    ) -> &mut Self {
         self.arc_consistency_checker(Some(Arc::new(checker)))
     }
 
+    /// Sets the consistency checker function to
+    /// [`crate::byte_equality_checker`]: the contents of all cache
+    /// hits must be bytewise identical, without considering any
+    /// metadata.
+    pub fn byte_equality_checker(&mut self) -> &mut Self {
+        self.consistency_checker(crate::byte_equality_checker)
+    }
+
+    /// Sets the consistency checker function to
+    /// [`crate::panicking_byte_equality_checker`]: the contents of
+    /// all cache hits must be bytewise identical, without considering
+    /// any metadata, and the call will panic on mismatch.
+    pub fn panicking_byte_equality_checker(&mut self) -> &mut Self {
+        self.consistency_checker(crate::panicking_byte_equality_checker)
+    }
+
     /// Removes the consistency checker function, if any.
-    pub fn clear_consistency_checker(self) -> Self {
+    pub fn clear_consistency_checker(&mut self) -> &mut Self {
         self.arc_consistency_checker(None)
     }
 
@@ -144,7 +160,7 @@ impl ReadOnlyCacheBuilder {
     /// [`ReadOnlyCacheBuilder::consistency_checker`].
     #[allow(clippy::type_complexity)] // We want the public type to be transparent
     pub fn arc_consistency_checker(
-        mut self,
+        &mut self,
         checker: Option<
             Arc<
                 dyn Fn(&mut File, &mut File) -> Result<()>
@@ -154,7 +170,7 @@ impl ReadOnlyCacheBuilder {
                     + std::panic::UnwindSafe,
             >,
         >,
-    ) -> Self {
+    ) -> &mut Self {
         self.consistency_checker = checker;
         self
     }
@@ -164,7 +180,7 @@ impl ReadOnlyCacheBuilder {
     ///
     /// Adds a plain cache directory if `num_shards <= 1`, and an
     /// actual sharded directory otherwise.
-    pub fn cache(self, path: impl AsRef<Path>, num_shards: usize) -> Self {
+    pub fn cache(&mut self, path: impl AsRef<Path>, num_shards: usize) -> &mut Self {
         if num_shards <= 1 {
             self.plain(path)
         } else {
@@ -176,7 +192,7 @@ impl ReadOnlyCacheBuilder {
     /// cache builder's search list.  A plain cache directory is
     /// merely a directory of files where the files' names match their
     /// key's name.
-    pub fn plain(mut self, path: impl AsRef<Path>) -> Self {
+    pub fn plain(&mut self, path: impl AsRef<Path>) -> &mut Self {
         self.stack.push(Box::new(PlainCache::new(
             path.as_ref().to_owned(),
             usize::MAX,
@@ -185,9 +201,23 @@ impl ReadOnlyCacheBuilder {
         self
     }
 
+    /// Adds a new plain cache directory for each path in `paths`.
+    /// The caches are appended in order to the end of the cache
+    /// builder's search list.
+    pub fn plain_caches<P>(&mut self, paths: impl IntoIterator<Item = P>) -> &mut Self
+    where
+        P: AsRef<Path>,
+    {
+        for path in paths {
+            self.plain(path);
+        }
+
+        self
+    }
+
     /// Adds a new sharded cache directory at `path` to the end of the
     /// cache builder's search list.
-    pub fn sharded(mut self, path: impl AsRef<Path>, num_shards: usize) -> Self {
+    pub fn sharded(&mut self, path: impl AsRef<Path>, num_shards: usize) -> &mut Self {
         self.stack.push(Box::new(ShardedCache::new(
             path.as_ref().to_owned(),
             num_shards,
@@ -196,6 +226,14 @@ impl ReadOnlyCacheBuilder {
         self
     }
 
+    /// Returns the contents of `self` as a fresh value; `self` is
+    /// reset to the default empty builder state.  This makes it
+    /// possible to declare simple configurations in a single
+    /// expression, with `.take().build()`.
+    pub fn take(&mut self) -> Self {
+        std::mem::take(self)
+    }
+
     /// Returns a fresh [`ReadOnlyCache`] for the builder's search list
     /// of constituent cache directories.
     pub fn build(self) -> ReadOnlyCache {
@@ -338,21 +376,9 @@ mod test {
     fn byte_equality_checker(
         counter: Arc<AtomicU64>,
     ) -> impl 'static + Fn(&mut File, &mut File) -> std::io::Result<()> {
-        use std::io::Read;
-
         move |x: &mut File, y: &mut File| {
-            let mut x_contents = Vec::new();
-            let mut y_contents = Vec::new();
-
             counter.fetch_add(1, Ordering::Relaxed);
-            x.read_to_end(&mut x_contents)?;
-            y.read_to_end(&mut y_contents)?;
-
-            if x_contents == y_contents {
-                Ok(())
-            } else {
-                Err(std::io::Error::new(std::io::ErrorKind::Other, "mismatch"))
-            }
+            crate::byte_equality_checker(x, y)
         }
     }
 
@@ -386,6 +412,7 @@ mod test {
             .plain(temp.path("first"))
             .plain(temp.path("second"))
             .consistency_checker(byte_equality_checker(counter.clone()))
+            .take()
             .build();
 
         let mut hit = ro
@@ -431,6 +458,7 @@ mod test {
             .plain(temp.path("first"))
             .plain(temp.path("second"))
             .consistency_checker(byte_equality_checker(counter))
+            .take()
             .build();
 
         // This call should error.
@@ -456,6 +484,7 @@ mod test {
             .plain(temp.path("second"))
             .consistency_checker(byte_equality_checker(counter.clone()))
             .clear_consistency_checker()
+            .take()
             .build();
 
         // This call should not error.
@@ -468,6 +497,81 @@ mod test {
         assert_eq!(counter.load(Ordering::Relaxed), 0);
     }
 
+    /// Populate two plain caches.  We should read from both.
+    #[test]
+    fn two_plain_caches() {
+        use test_dir::{DirBuilder, FileType, TestDir};
+
+        let temp = TestDir::temp()
+            .create("first", FileType::Dir)
+            .create("second", FileType::Dir)
+            .create("first/0", FileType::ZeroFile(2))
+            .create("second/1", FileType::ZeroFile(3));
+
+        let ro = ReadOnlyCacheBuilder::new()
+            .plain_caches(["first", "second"].iter().map(|p| temp.path(p)))
+            .take()
+            .build();
+
+        // We should find 0 and 1.
+        let _ = ro
+            .get(&TestKey::new("0"))
+            .expect("must succeed")
+            .expect("must exist");
+
+        let _ = ro
+            .get(&TestKey::new("1"))
+            .expect("must succeed")
+            .expect("must exist");
+
+        // But not 2.
+        assert!(ro.get(&TestKey::new("2")).expect("must succeed").is_none());
+    }
+
+    /// Use a byte equality checker with two different cache files for
+    /// the same key.  We should find an error.
+    #[test]
+    fn test_byte_equality_checker() {
+        use test_dir::{DirBuilder, FileType, TestDir};
+
+        let temp = TestDir::temp()
+            .create("first", FileType::Dir)
+            .create("second", FileType::Dir)
+            .create("first/0", FileType::ZeroFile(2))
+            .create("second/0", FileType::ZeroFile(3));
+
+        let ro = ReadOnlyCacheBuilder::new()
+            .plain_caches(["first", "second"].iter().map(|p| temp.path(p)))
+            .byte_equality_checker()
+            .take()
+            .build();
+
+        assert!(ro.get(&TestKey::new("0")).is_err());
+    }
+
+    /// Use a panicking byte equality checker with two different cache
+    /// files for the same key.  We should find an error.
+    #[test]
+    #[should_panic(expected = "file contents do not match")]
+    fn test_panicking_byte_equality_checker() {
+        use test_dir::{DirBuilder, FileType, TestDir};
+
+        let temp = TestDir::temp()
+            .create("first", FileType::Dir)
+            .create("second", FileType::Dir)
+            .create("first/0", FileType::ZeroFile(2))
+            .create("second/0", FileType::ZeroFile(3));
+
+        let ro = ReadOnlyCacheBuilder::new()
+            .plain_caches(["first", "second"].iter().map(|p| temp.path(p)))
+            .panicking_byte_equality_checker()
+            .take()
+            .build();
+
+        // We should fail before returning Err.
+        assert!(ro.get(&TestKey::new("0")).is_ok());
+    }
+
     /// Populate a plain and a sharded cache. We should be able to access
     /// both.
     #[test]
@@ -534,6 +638,7 @@ mod test {
             let ro = ReadOnlyCacheBuilder::new()
                 .sharded(temp.path("sharded"), 10)
                 .plain(temp.path("plain"))
+                .take()
                 .build();
 
             assert!(matches!(ro.get(&TestKey::new("Missing")), Ok(None)));
@@ -579,6 +684,7 @@ mod test {
             let ro = ReadOnlyCacheBuilder::new()
                 .cache(temp.path("plain"), 1)
                 .cache(temp.path("sharded"), 10)
+                .take()
                 .build();
 
             {
