@@ -221,7 +221,7 @@ impl CacheBuilder {
     ///
     /// Kismet will propagate the error on mismatch.
     pub fn consistency_checker(
-        self,
+        &mut self,
         checker: impl Fn(&mut File, &mut File) -> Result<()>
             + Sync
             + Send
@@ -229,12 +229,28 @@ impl CacheBuilder {
             + std::panic::UnwindSafe
             + Sized
             + 'static,
-    ) -> Self {
+    ) -> &mut Self {
         self.arc_consistency_checker(Some(Arc::new(checker)))
     }
 
+    /// Sets the consistency checker function to
+    /// [`crate::byte_equality_checker`]: the contents of all cache
+    /// hits must be bytewise identical, without considering any
+    /// metadata.
+    pub fn byte_equality_checker(&mut self) -> &mut Self {
+        self.consistency_checker(crate::byte_equality_checker)
+    }
+
+    /// Sets the consistency checker function to
+    /// [`crate::panicking_byte_equality_checker`]: the contents of
+    /// all cache hits must be bytewise identical, without considering
+    /// any metadata, and the call will panic on mismatch.
+    pub fn panicking_byte_equality_checker(&mut self) -> &mut Self {
+        self.consistency_checker(crate::panicking_byte_equality_checker)
+    }
+
     /// Removes the consistency checker function, if any.
-    pub fn clear_consistency_checker(self) -> Self {
+    pub fn clear_consistency_checker(&mut self) -> &mut Self {
         self.arc_consistency_checker(None)
     }
 
@@ -242,7 +258,7 @@ impl CacheBuilder {
     /// checker function.  See [`CacheBuilder::consistency_checker`].
     #[allow(clippy::type_complexity)] // We want the public type to be transparent
     pub fn arc_consistency_checker(
-        mut self,
+        &mut self,
         checker: Option<
             Arc<
                 dyn Fn(&mut File, &mut File) -> Result<()>
@@ -252,9 +268,9 @@ impl CacheBuilder {
                     + std::panic::UnwindSafe,
             >,
         >,
-    ) -> Self {
+    ) -> &mut Self {
         self.consistency_checker = checker.clone();
-        self.read_side = self.read_side.arc_consistency_checker(checker);
+        self.read_side.arc_consistency_checker(checker);
         self
     }
 
@@ -262,7 +278,12 @@ impl CacheBuilder {
     ///
     /// The read-write cache will be a plain cache directory if
     /// `num_shards <= 1`, and a sharded directory otherwise.
-    pub fn writer(self, path: impl AsRef<Path>, num_shards: usize, total_capacity: usize) -> Self {
+    pub fn writer(
+        &mut self,
+        path: impl AsRef<Path>,
+        num_shards: usize,
+        total_capacity: usize,
+    ) -> &mut Self {
         if num_shards <= 1 {
             self.plain_writer(path, total_capacity)
         } else {
@@ -272,7 +293,7 @@ impl CacheBuilder {
 
     /// Sets the read-write cache directory to a plain directory at
     /// `path`, with a target file count of up to `capacity`.
-    pub fn plain_writer(mut self, path: impl AsRef<Path>, capacity: usize) -> Self {
+    pub fn plain_writer(&mut self, path: impl AsRef<Path>, capacity: usize) -> &mut Self {
         self.write_side.insert(Arc::new(PlainCache::new(
             path.as_ref().to_owned(),
             capacity,
@@ -284,11 +305,11 @@ impl CacheBuilder {
     /// `path`, with `num_shards` subdirectories and a target file
     /// count of up to `capacity` for the entire cache.
     pub fn sharded_writer(
-        mut self,
+        &mut self,
         path: impl AsRef<Path>,
         num_shards: usize,
         total_capacity: usize,
-    ) -> Self {
+    ) -> &mut Self {
         self.write_side.insert(Arc::new(ShardedCache::new(
             path.as_ref().to_owned(),
             num_shards,
@@ -312,7 +333,7 @@ impl CacheBuilder {
     /// e.g., via
     /// [tmpfiles.d](https://www.freedesktop.org/software/systemd/man/tmpfiles.d.html),
     /// or tagged with a [boot id](https://man7.org/linux/man-pages/man3/sd_id128_get_machine.3.html).
-    pub fn auto_sync(mut self, sync: bool) -> Self {
+    pub fn auto_sync(&mut self, sync: bool) -> &mut Self {
         self.auto_sync = sync;
         self
     }
@@ -322,25 +343,44 @@ impl CacheBuilder {
     ///
     /// Adds a plain cache directory if `num_shards <= 1`, and a sharded
     /// directory otherwise.
-    pub fn reader(mut self, path: impl AsRef<Path>, num_shards: usize) -> Self {
-        self.read_side = self.read_side.cache(path, num_shards);
+    pub fn reader(&mut self, path: impl AsRef<Path>, num_shards: usize) -> &mut Self {
+        self.read_side.cache(path, num_shards);
         self
     }
 
     /// Adds a new plain (unsharded) read-only cache directory at
     /// `path` to the end of the cache builder's search list.
-    pub fn plain_reader(mut self, path: impl AsRef<Path>) -> Self {
-        self.read_side = self.read_side.plain(path);
+    pub fn plain_reader(&mut self, path: impl AsRef<Path>) -> &mut Self {
+        self.read_side.plain(path);
+        self
+    }
+
+    /// Adds a new plain cache read-only directory for each path in
+    /// `paths`.  The caches are appended in order to the end of the
+    /// cache builder's search list.
+    pub fn plain_readers(
+        &mut self,
+        paths: impl IntoIterator<Item = impl AsRef<Path>>,
+    ) -> &mut Self {
+        self.read_side.plain_caches(paths);
         self
     }
 
     /// Adds a new sharded read-only cache directory at `path` to the
     /// end of the cache builder's search list.
-    pub fn sharded_reader(mut self, path: impl AsRef<Path>, num_shards: usize) -> Self {
-        self.read_side = self.read_side.sharded(path, num_shards);
+    pub fn sharded_reader(&mut self, path: impl AsRef<Path>, num_shards: usize) -> &mut Self {
+        self.read_side.sharded(path, num_shards);
         self
     }
 
+    /// Returns the contents of `self` as a fresh value; `self` is
+    /// reset to the default empty builder state.  This makes it
+    /// possible to declare simple configurations in a single
+    /// expression, with `.take().build()`.
+    pub fn take(&mut self) -> Self {
+        std::mem::take(self)
+    }
+
     /// Returns a fresh [`Cache`] for the builder's write cache and
     /// additional search list of read-only cache directories.
     pub fn build(self) -> Cache {
@@ -441,7 +481,9 @@ impl Cache {
     /// a file in all cases (unless the call fails with an error).
     ///
     /// Always invokes `populate` for a consistency check when a
-    /// consistency check function is provided.
+    /// consistency check function is provided.  The `populate`
+    /// function can return `ErrorKind::NotFound` to skip the
+    /// comparison without failing the whole call.
     ///
     /// Fails with [`ErrorKind::InvalidInput`] if `key.name` is
     /// invalid (empty, or starts with a dot or a forward or back slash).
@@ -465,7 +507,9 @@ impl Cache {
     /// `judge` to determine what to do with the hit.
     ///
     /// Always invokes `populate` for a consistency check when a
-    /// consistency check function is provided.
+    /// consistency check function is provided.  The `populate`
+    /// function can return `ErrorKind::NotFound` to skip the
+    /// comparison without failing the whole call.
     ///
     /// Fails with [`ErrorKind::InvalidInput`] if `key.name` is
     /// invalid (empty, or starts with a dot or a forward or back slash).
@@ -542,7 +586,12 @@ impl Cache {
 
                     if let Some(checker) = self.consistency_checker.as_ref() {
                         let mut tmp = get_tempfile()?;
-                        populate(&mut tmp, None)?;
+                        match populate(&mut tmp, None) {
+                            Err(e) if e.kind() == ErrorKind::NotFound => {
+                                return Ok(file);
+                            }
+                            ret => ret?,
+                        };
                         tmp.seek(SeekFrom::Start(0))?;
                         checker(&mut file, &mut tmp)?;
                         file.seek(SeekFrom::Start(0))?;
@@ -560,7 +609,13 @@ impl Cache {
                     if let Some(checker) = self.consistency_checker.as_ref() {
                         let mut tmp = get_tempfile()?;
 
-                        populate(&mut tmp, None)?;
+                        match populate(&mut tmp, None) {
+                            Err(e) if e.kind() == ErrorKind::NotFound => {
+                                return Ok(file);
+                            }
+                            ret => ret?,
+                        };
+
                         tmp.seek(SeekFrom::Start(0))?;
                         checker(&mut file, &mut tmp)?;
                         file.seek(SeekFrom::Start(0))?;
@@ -799,21 +854,9 @@ mod test {
     fn byte_equality_checker(
         counter: Arc<AtomicU64>,
     ) -> impl 'static + Fn(&mut File, &mut File) -> std::io::Result<()> {
-        use std::io::Read;
-
         move |x: &mut File, y: &mut File| {
-            let mut x_contents = Vec::new();
-            let mut y_contents = Vec::new();
-
             counter.fetch_add(1, Ordering::Relaxed);
-            x.read_to_end(&mut x_contents)?;
-            y.read_to_end(&mut y_contents)?;
-
-            if x_contents == y_contents {
-                Ok(())
-            } else {
-                Err(std::io::Error::new(std::io::ErrorKind::Other, "mismatch"))
-            }
+            crate::byte_equality_checker(x, y)
         }
     }
 
@@ -845,7 +888,7 @@ mod test {
     // input file does not exist.
     #[test]
     fn empty_no_auto_sync() {
-        let cache = CacheBuilder::new().auto_sync(false).build();
+        let cache = CacheBuilder::new().auto_sync(false).take().build();
 
         assert!(matches!(cache.get(&TestKey::new("foo")), Ok(None)));
         assert!(matches!(
@@ -868,6 +911,8 @@ mod test {
     /// should access both.
     #[test]
     fn consistency_checker_success() {
+        use std::io::Error;
+        use std::io::ErrorKind;
         use std::io::Read;
         use std::io::Write;
         use test_dir::{DirBuilder, FileType, TestDir};
@@ -879,7 +924,8 @@ mod test {
             .create("second/0", FileType::ZeroFile(2))
             .create("first/1", FileType::ZeroFile(1))
             .create("second/2", FileType::ZeroFile(3))
-            .create("second/3", FileType::ZeroFile(3));
+            .create("second/3", FileType::ZeroFile(3))
+            .create("second/4", FileType::ZeroFile(4));
 
         let counter = Arc::new(AtomicU64::new(0));
 
@@ -887,6 +933,7 @@ mod test {
             .plain_writer(temp.path("first"), 100)
             .plain_reader(temp.path("second"))
             .consistency_checker(byte_equality_checker(counter.clone()))
+            .take()
             .build();
 
         // Find a hit in both caches. The checker should be invoked.
@@ -905,14 +952,34 @@ mod test {
 
         // Do the same via `ensure`.
         {
+            counter.store(0, Ordering::Relaxed);
             let mut populated = cache
-                .ensure(&TestKey::new("0"), |dst: &mut File| {
+                .ensure(&TestKey::new("0"), |dst| {
                     dst.write_all("00".as_bytes())?;
                     Ok(())
                 })
                 .expect("ensure must succeed");
 
-            assert_eq!(counter.load(Ordering::Relaxed), 3);
+            assert_eq!(counter.load(Ordering::Relaxed), 2);
+
+            let mut contents = Vec::new();
+            populated
+                .read_to_end(&mut contents)
+                .expect("read should succeed");
+            assert_eq!(contents, "00".as_bytes());
+        }
+
+        // Now return `NotFound` from the `populate` callback,
+        // we should still succeed.
+        {
+            counter.store(0, Ordering::Relaxed);
+            let mut populated = cache
+                .ensure(&TestKey::new("0"), |_| {
+                    Err(Error::new(ErrorKind::NotFound, "not found"))
+                })
+                .expect("ensure must succeed");
+
+            assert_eq!(counter.load(Ordering::Relaxed), 1);
 
             let mut contents = Vec::new();
             populated
@@ -996,6 +1063,26 @@ mod test {
             assert_eq!(contents, "000".as_bytes());
         }
 
+        // Again, but now the `populate` callback returns `NotFound`.
+        {
+            counter.store(0, Ordering::Relaxed);
+            let mut populated = cache
+                .get_or_update(
+                    &TestKey::new("4"),
+                    |_| CacheHitAction::Accept,
+                    |_, _| Err(Error::new(ErrorKind::NotFound, "not found")),
+                )
+                .expect("get_or_update must succeed");
+
+            assert_eq!(counter.load(Ordering::Relaxed), 0);
+
+            let mut contents = Vec::new();
+            populated
+                .read_to_end(&mut contents)
+                .expect("read should succeed");
+            assert_eq!(contents, "0000".as_bytes());
+        }
+
         // Make sure we succeed on plain misses.
         {
             counter.store(0, Ordering::Relaxed);
@@ -1040,6 +1127,7 @@ mod test {
             .plain_writer(temp.path("first"), 100)
             .plain_reader(temp.path("second"))
             .consistency_checker(byte_equality_checker(counter))
+            .take()
             .build();
 
         // This call should error.
@@ -1102,6 +1190,7 @@ mod test {
             .plain_reader(temp.path("second"))
             .consistency_checker(byte_equality_checker(counter.clone()))
             .clear_consistency_checker()
+            .take()
             .build();
 
         // This call should not error.
@@ -1142,6 +1231,37 @@ mod test {
         assert_eq!(counter.load(Ordering::Relaxed), 0);
     }
 
+    /// Populate two plain caches.  We should read from both.
+    #[test]
+    fn two_plain_caches() {
+        use test_dir::{DirBuilder, FileType, TestDir};
+
+        let temp = TestDir::temp()
+            .create("first", FileType::Dir)
+            .create("second", FileType::Dir)
+            .create("first/0", FileType::ZeroFile(2))
+            .create("second/1", FileType::ZeroFile(3));
+
+        let ro = CacheBuilder::new()
+            .plain_readers(["first", "second"].iter().map(|p| temp.path(p)))
+            .take()
+            .build();
+
+        // We should find 0 and 1.
+        let _ = ro
+            .get(&TestKey::new("0"))
+            .expect("must succeed")
+            .expect("must exist");
+
+        let _ = ro
+            .get(&TestKey::new("1"))
+            .expect("must succeed")
+            .expect("must exist");
+
+        // But not 2.
+        assert!(ro.get(&TestKey::new("2")).expect("must succeed").is_none());
+    }
+
     // Fail to find a file, ensure it, then see that we can get it.
     #[test]
     fn test_ensure() {
@@ -1153,6 +1273,7 @@ mod test {
         let cache = CacheBuilder::new()
             .writer(temp.path("."), 1, 10)
             .auto_sync(false)
+            .take()
             .build();
         let key = TestKey::new("foo");
 
@@ -1223,6 +1344,7 @@ mod test {
         let cache = CacheBuilder::new()
             .writer(temp.path("cache"), 1, 10)
             .plain_reader(temp.path("extra_plain"))
+            .take()
             .build();
         let key = TestKey::new("foo");
 
@@ -1255,6 +1377,7 @@ mod test {
         {
             let new_cache = CacheBuilder::new()
                 .writer(temp.path("cache"), 1, 10)
+                .take()
                 .build();
             let mut fetched = new_cache
                 .get(&key)
@@ -1296,6 +1419,7 @@ mod test {
             // Make it sharded, because why not?
             .writer(temp.path("cache"), 2, 10)
             .plain_reader(temp.path("extra_plain"))
+            .take()
             .build();
         let key = TestKey::new("foo");
         let key2 = TestKey::new("bar");
@@ -1359,6 +1483,7 @@ mod test {
         {
             let new_cache = CacheBuilder::new()
                 .writer(temp.path("cache"), 2, 10)
+                .take()
                 .build();
 
             // The new cache shouldn't have the old key.
@@ -1405,6 +1530,7 @@ mod test {
             // Make it sharded, because why not?
             .writer(temp.path("cache"), 2, 10)
             .plain_reader(temp.path("extra_plain"))
+            .take()
             .build();
         let key = TestKey::new("foo");
 
@@ -1502,6 +1628,7 @@ mod test {
         {
             let new_cache = CacheBuilder::new()
                 .writer(temp.path("cache"), 2, 10)
+                .take()
                 .build();
 
             // But it should have `key2`.
@@ -1542,6 +1669,7 @@ mod test {
 
         let cache = CacheBuilder::new()
             .plain_reader(temp.path("extra_plain"))
+            .take()
             .build();
         let key = TestKey::new("foo");
         let key2 = TestKey::new("bar");
@@ -1600,6 +1728,51 @@ mod test {
             assert_eq!(&dst, b"updated2");
         }
     }
+
+    /// Use a byte equality checker with two different cache files for
+    /// the same key.  We should find an error.
+    #[test]
+    fn test_byte_equality_checker() {
+        use test_dir::{DirBuilder, FileType, TestDir};
+
+        let temp = TestDir::temp()
+            .create("first", FileType::Dir)
+            .create("second", FileType::Dir)
+            .create("first/0", FileType::ZeroFile(2))
+            .create("second/0", FileType::ZeroFile(3));
+
+        let cache = CacheBuilder::new()
+            .plain_readers(["first", "second"].iter().map(|p| temp.path(p)))
+            .byte_equality_checker()
+            .take()
+            .build();
+
+        assert!(cache.get(&TestKey::new("0")).is_err());
+    }
+
+    /// Use a panicking byte equality checker with two different cache
+    /// files for the same key.  We should find an error.
+    #[test]
+    #[should_panic(expected = "file contents do not match")]
+    fn test_panicking_byte_equality_checker() {
+        use test_dir::{DirBuilder, FileType, TestDir};
+
+        let temp = TestDir::temp()
+            .create("first", FileType::Dir)
+            .create("second", FileType::Dir)
+            .create("first/0", FileType::ZeroFile(2))
+            .create("second/0", FileType::ZeroFile(3));
+
+        let cache = CacheBuilder::new()
+            .plain_readers(["first", "second"].iter().map(|p| temp.path(p)))
+            .panicking_byte_equality_checker()
+            .take()
+            .build();
+
+        // We should fail before returning Err.
+        assert!(cache.get(&TestKey::new("0")).is_ok());
+    }
+
     // Smoke test a wrapped plain cache.
     #[test]
     fn smoke_test_plain() {
@@ -1635,6 +1808,7 @@ mod test {
         let cache = CacheBuilder::new()
             .writer(temp.path("cache"), 1, 10)
             .reader(temp.path("extra"), 1)
+            .take()
             .build();
 
         // There shouldn't be anything for "a"
@@ -1779,6 +1953,7 @@ mod test {
             .writer(temp.path("cache"), 10, 10)
             .plain_reader(temp.path("extra_plain"))
             .sharded_reader(temp.path("extra_sharded"), 10)
+            .take()
             .build();
 
         // There shouldn't be anything for "a"
